/**
 *
 * OFDevCon Example Code Sprint
 * 
 * This example shows building a mesh, texturing it with a webcam, and extruding the vertices based on the pixel brightness
 * Moving the mouse also rotates the mesh to see it at different angles
 *
 * Created by Tim Gfrerer and James George for openFrameworks workshop at Waves Festival Vienna sponsored by Lichterloh and Pratersauna
 * Adapted during ofDevCon on 2/23/2012
 */


#include "ofApp.h"
#define POINT_STEP 2
#define POINTSIZE 2

float nFocalLengthW = 333.0f;
float nFocalLengthH = 350.0f;

//--------------------------------------------------------------
void ofApp::setup(){

	ofSetVerticalSync(true);
	ofSetFrameRate(30);
	ofBackground(66,66,66);
	glEnable(GL_DEPTH_TEST);
	//initialize the video grabber

	tCam.camWidth = 320;
	tCam.camHeight = 240;
	tCam.start(0, false);

	//store the width and height for convenience
	int width = tCam.camWidth;
	int height = tCam.camHeight;
	
	//add one vertex to the mesh for each pixel
	for (int y = 0; y < height; y++){
		for (int x = 0; x<width; x++){
			mainMesh.addVertex(ofPoint(x,y,0));	// mesh index = x + y*width
												// this replicates the pixel array within the camera bitmap...
			mainMesh.addColor(ofFloatColor(0,0,0));  // placeholder for colour data, we'll get this from the camera
		}
	}
	
	for (int y = 0; y<height-1; y++){
		for (int x=0; x<width-1; x++){
			mainMesh.addIndex(x+y*width);				// 0
			mainMesh.addIndex((x+1)+y*width);			// 1
			mainMesh.addIndex(x+(y+1)*width);			// 10
			
			mainMesh.addIndex((x+1)+y*width);			// 1
			mainMesh.addIndex((x+1)+(y+1)*width);		// 11
			mainMesh.addIndex(x+(y+1)*width);			// 10
		}
	}
	myVbo = ofMesh::plane(tCam.camWidth/ POINT_STEP, tCam.camHeight/ POINT_STEP, tCam.camWidth/ POINT_STEP, tCam.camHeight/ POINT_STEP, OF_PRIMITIVE_TRIANGLES);
	//vector<ofVec3f> verts = myVbo.getNumVertices();
	for (int i = 0; i < myVbo.getNumVertices(); i++) {
		ofVec3f vert = myVbo.getVertex(i);
		myVbo.setTexCoord(i, ofVec2f(vert.x, vert.y));
		myVbo.addColor(ofFloatColor(0, 0, 0, 0));
	}

	//this is an annoying thing that is used to flip the camera
	cam.setScale(1,-1,1);
	cam.setDistance(600);	

	//this determines how much we push the meshes out
	extrusionAmount = 300.0;
}

//--------------------------------------------------------------
void ofApp::update(){
	//grab a new frame


	//update the mesh if we have a new frame
	if (tCam.isFrameNew(0)){
		unsigned char *pColor = tCam.getImage(0);
		WORD *pDepth = (WORD*)tCam.getDepth(0);

		//this determines how far we extrude the mesh
		for (int y = 0; y < tCam.camHeight; y+= POINT_STEP)
		{
		
			for (int x = 0; x < tCam.camWidth; x+= POINT_STEP)
			{
				int p = y*tCam.camWidth + x;
				int pp = (int)(y/POINT_STEP)*(int)(tCam.camWidth/POINT_STEP) + (int)x/POINT_STEP;
//				int pc = (tCam.camHeight - y - 1)*tCam.camWidth + x;
				int pc = y *tCam.camWidth + x;
				ofFloatColor sampleColor(pColor[pc * 3+0] / 255.f,				// r
					pColor[pc * 3 + 1] / 255.f,			// g
					pColor[pc * 3 + 2] / 255.f);			// b

				//now we get the vertex aat this position
				//we extrude the mesh based on it's brightness
				float zz = (float)pDepth[p];
				float xx = (float)(x - tCam.camWidth/2)*zz / nFocalLengthW;
				float yy = (float)(y - tCam.camHeight/2)*zz / nFocalLengthH;

				ofVec3f tmpVec(xx, yy, -zz);
//				tmpVec = mainMesh.getVertex(p);
//				tmpVec.z = -zz;
				myVbo.setVertex(pp, tmpVec);
				myVbo.setColor(pp, sampleColor);
			}
		}
	}
	
	//let's move the camera when you move the mouse
	float rotateAmount = ofMap(ofGetMouseY(), 0, ofGetHeight(), 0, 360);

	
	//move the camera around the mesh
	ofVec3f camDirection(0,0,1);
	ofVec3f centre(tCam.camWidth/2.f,tCam.camHeight/2.f, 255/2.f);
	ofVec3f camDirectionRotated = camDirection.getRotated(rotateAmount, ofVec3f(1,0,0));
	ofVec3f camPosition = centre + camDirectionRotated * extrusionAmount;
	
//	cam.setPosition(camPosition);
	cam.lookAt(centre);
}

//--------------------------------------------------------------
void ofApp::draw(){
	//we have to disable depth testing to draw the video frame
	ofDisableDepthTest();
//	vidGrabber.draw(20,20);
	
	//but we want to enable it to show the mesh
	ofEnableDepthTest();
	cam.setScale(1,-1,1);
	cam.begin();		
	ofTranslate(ofGetWidth()/2,ofGetHeight()/2);

	glPointSize(POINTSIZE);
	//You can either draw the mesh or the wireframe
	// mainMesh.drawWireframe();
//	mainMesh.drawFaces();
	myVbo.drawVertices();

	cam.end();
	
	//draw framerate for fun
	ofSetColor(255);
	string msg = "fps: " + ofToString(ofGetFrameRate(), 2);
	ofDrawBitmapString(msg, 10, 20);
	
}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){
	switch(key) {
		case 'f':
			ofToggleFullscreen();
			break;
	}

}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){

}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y ){
	
}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){


}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseEntered(int x, int y){

}

//--------------------------------------------------------------
void ofApp::mouseExited(int x, int y){

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){ 

}
